/*
*  Rubrica
*  file: file.c
*  
*  Copyright (C) 2000-2002 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>
#include <gnome.h>
#include <libxml/tree.h>
#include <libxml/parser.h>

#include "rubrica_type.h"
#include "rubrica.h"
#include "card.h"
#include "card_item.h"
#include "form_card.h"
#include "prefer.h"
#include "main.h"
#include "file.h"
#include "file_card.h"
#include "import_int.h"
#include "import.h"
#include "export.h"
#include "utils.h"
#include "dictionary.h"
#include "dialog.h"
#include "ui.h"
#include "log.h"
#include "data_view.h"

#define DEBUG(rubrica, item)   {                                  \
                    g_print("\n\t\tALLOCO ELEMENTO");             \
                    rubrica_item_print_debug((rubrica), (item));  \
                    }

/*  open the file "filename"
    return ptr to xml doc or NULL on error
 */
xmlDocPtr 
rubrica_file_open_doc(gchar *filename)
{
  xmlDocPtr doc;

  g_return_val_if_fail(filename != NULL, FALSE);
  
  /* read and parse file 
   */
  doc = xmlParseFile(filename);

  if (!doc)
    {
      rubrica_dialog_error_loading_file(filename, RE_PARSE_XML_DOC);
      
      return NULL;
    }  

  return doc;
}

/*  check if file "filename" is a rubrica doc
    return TRUE if xml doc is a rubrica doc FALSE otherwise
*/
gboolean 
rubrica_file_is_rubrica_doc(xmlDocPtr doc, gchar *filename)
{
  g_return_val_if_fail(doc != NULL, FALSE);

  if (!doc->children || 
      !doc->children->name || 
      g_ascii_strcasecmp(doc->children->name, "Rubrica") != 0)
    {      
      rubrica_dialog_error_loading_file(filename, 
					RE_XML_DOC_NO_RUBRICA_HEADER);   
      xmlFreeDoc(doc);

      return FALSE;
    }  

  return TRUE;
}

/*  get the file format of the rubrica's doc    
*/
gint 
rubrica_file_get_fileformat(xmlDocPtr doc)
{
  if(!xmlHasProp(doc->children, "fileformat"))
    return rubrica_utils_str2int(NULL);

  return rubrica_utils_str2int(xmlGetProp(doc->children, "fileformat"));
}

/*  get version.
    the version of rubrica that make the file
*/
gchar*
rubrica_file_get_version(xmlDocPtr doc)
{
  if(!xmlHasProp(doc->children, "version"))
    return NULL;

  return xmlGetProp(doc->children, "version");
}


/*  get the document type
    values: ADDRESSBOOK_TYPE_DOC
            ACCOUNT_TYE_DOC
*/
gchar*
rubrica_file_get_document_tpye(xmlDocPtr doc)
{
  if(!xmlHasProp(doc->children, "doctype"))
    return NULL;

  return xmlGetProp(doc->children, "doctype");
}


/*  check the "filename" extension
    return TRUE  if filename has extension .rub FALSE otherwise
*/
gboolean 
rubrica_file_has_rub_extension(gchar *filename)
{
  gchar *tmp=NULL, *buf;
  gint i, len;

  i = 0;
  buf = g_path_get_basename(filename);
  len = strlen(buf);

  while(*buf != '.')
    {
      buf++;
      i++;
            
      if (i == len)     /* end buffer was reached and dot wasn't found */
	return FALSE;	
    }

  tmp = (gchar*) g_malloc(((len - i)*sizeof(gchar))+1);
  strcpy(tmp, buf);
  
  if ((strcmp(tmp,".rub")) == 0)
    {
      g_free(tmp);
      return TRUE;
    }
  else 
    g_free(tmp);

  g_free(buf);
  return FALSE;  
}

/*  import an old rubrica doc
*/
gboolean 
rubrica_file_import_old_file(gchar* fname)
{
  gchar* msg;
  gchar* stylesheet;
  gchar* filename;
  
  rubrica_log_write_log(_("\nImporting old rubrica document. Please wait..."));
  msg = g_strdup_printf(_("\nThe document %s will be moved to "
			  "file format #%d"), fname, RUBRICA_FILE_FORMAT); 
  rubrica_log_write_log(msg);
  g_free(msg);
  
  filename = g_strdup_printf("%s%s.tmpfile",
			     rubrica_preferences_get_rubrica_home(),
			     G_DIR_SEPARATOR_S);
  
  stylesheet = g_strdup_printf("%s%sstylesheet%srub2rub.xsl", 
			       RUBRICA_SCRIPT_DIR, 
			       G_DIR_SEPARATOR_S,
			       G_DIR_SEPARATOR_S); 
  
  if (!g_file_test(stylesheet, G_FILE_TEST_EXISTS))
    {
      rubrica_dialog_error(_("Can't find the stylesheet."
			     "\nI can't read the old file."));
      
      g_free(stylesheet);
      g_free(filename);
      
      return FALSE;
    }        
  
  if(!rubrica_export_run_engine(fname, filename, stylesheet))
    {
      /* error dialog shown in rubrica_export_engine */
      g_free(stylesheet);
      g_free(filename);
      
      return FALSE;
    }
  else
    {
      gchar *newname;

      newname = g_strdup_printf("%s.SAVE", fname);
      if (rename(fname, newname) == -1)
	{
	  msg = g_strdup_printf("\nRubrica ** exporting %s to %s failed", 
				fname, newname);
	  
	  rubrica_log_write_log(msg);
	  g_free(msg);
	  
	  return FALSE;
	}
      g_free(newname);
      
      remove(fname);
      if (-1 == rename(filename, fname))
	{
	  gchar *msg;
	  
	  msg = g_strdup_printf("\nRubrica ** renaming %s to %s failed", 
				filename, fname);
	  
	  rubrica_log_write_log(msg);
	  g_free(msg);
	  
	  return FALSE;
	}
      g_free(filename);
    }

  return TRUE;
}



/*
 *
 *  Backup routines
 *
 */ 
void 
rubrica_file_do_backup(RubricaDoc *doc, gchar *oldname)
{
  gint err;
  gchar *newname;

  if (backup_is_increasing())
    {
      newname = g_strdup_printf("%s1", oldname);
      
      /*  if exist a file with same name, then do a backup of old file 
       */
      if (g_file_test(newname, G_FILE_TEST_EXISTS))
	{
	  gchar *path;
	  gchar *ext=NULL, *f=NULL;
	  gchar *bfile;
	  gint num;

	  path = g_path_get_dirname(newname);
	  rubrica_utils_file_split(newname, &f, &ext, &num);
	  
	  bfile = g_strdup_printf("%s/%s%d.%s", path, f, num, ext);
	  rubrica_file_do_backup(doc, bfile);

	  g_free(bfile);
	}
      else
	{
	  err = rename(oldname, newname);  
	  
	  if (!err)
	    {
	      gchar *str;

	      str = g_strdup_printf("\nBackup -- old file: %s has "
				    "been saved in: %s", oldname, newname);
	      
	      rubrica_log_write_log(str);
	      g_free(str);
	    }
	      
	  else
	    {
	      gchar *str;

	      str = g_strdup_printf("\nBackup -- This error occurred "
				    "during backup\nold file name: %s\n"
				    "new file name: %s\nerror: %s", 
				    oldname, newname, strerror(errno)); 
	     
	      rubrica_log_write_log(str);
	      g_free(str);
	    }	      
	}
      
      g_free(newname);
    }
  else
    rubrica_file_do_bak_backup(doc, oldname);
}


void 
rubrica_file_do_bak_backup(RubricaDoc *doc, gchar *fname)
{
  gchar *ext=NULL, *file=NULL, *new;
  gchar *path;
  gint num, err;
  
  path = g_path_get_dirname(fname);
  rubrica_utils_file_split(fname, &file, &ext, &num);	  
  
  new = g_strdup_printf("%s/%s.bak", path, file);
  err = rename(fname, new);  
  
  if (!err)
    {
      gchar *str;
      
      str = g_strdup_printf("\nBackup -- old file: %s has been saved in: %s", 
			    fname, new);
      
      rubrica_log_write_log(str);
      g_free(str);
    }
  
  g_free(new);  
}



/* File Selection Dialogs
 */
void 
rubrica_file_load_file(void)
{
  GtkWidget* fs;
  GtkWidget* mainbox;
  GtkWidget* hbox;
  GtkWidget* label;
  GtkWidget* optmenu;
  GtkWidget* app;

  app = rubrica_get_app();

  fs = gtk_file_selection_new(_("Open file..."));
  mainbox = GTK_FILE_SELECTION (fs)->main_vbox; 
  
  hbox = gtk_hbox_new(FALSE, 12);
  gtk_box_pack_start(GTK_BOX(mainbox), hbox, FALSE, FALSE, 12);
  gtk_widget_show(hbox);
  
  label = gtk_label_new(_("<b>File type</b>"));
  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);    
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
  gtk_widget_show(label);

  optmenu = rubrica_import_build_options();
  gtk_box_pack_start(GTK_BOX(hbox), optmenu, TRUE, TRUE, 0);
  gtk_option_menu_set_history(GTK_OPTION_MENU(optmenu), OPEN_RUBRICA); 

  if (rubrica_preferences_get_have_default_folder())
    {
      gchar *path;
         
      path = rubrica_preferences_get_default_folder();
      if (path)
	{
	  gtk_file_selection_set_filename (GTK_FILE_SELECTION(fs), path);      
	  
	  g_free(path);
	}
    }
 
  /* connetto i bottoni ok e cancel */
  g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(fs)->ok_button), "clicked",
		   G_CALLBACK(rubrica_file_open_file), (gpointer) fs);

  g_signal_connect_swapped(G_OBJECT(GTK_FILE_SELECTION(fs)->cancel_button),
			   "clicked", G_CALLBACK(gtk_widget_destroy), 
			   (gpointer)fs);

  /* visualizzo la finestra in posizione centrale */
  gtk_window_set_position(GTK_WINDOW(fs), GTK_WIN_POS_CENTER);

  /* il genitore della finestra fileselection: app */
  gtk_window_set_transient_for(GTK_WINDOW(fs), GTK_WINDOW(app));

  gtk_widget_show(GTK_WIDGET(fs));
}


void 
rubrica_file_save_as(RubricaDoc *doc)
{
  RubricaDocType doctype;
  GtkWidget *app;
  GtkWidget *fs;
  GtkWidget* mainbox;
  GtkWidget* hbox;
  GtkWidget* label;
  GtkWidget* optmenu;
  gchar *filename; 
  
  app      = rubrica_get_app();
  filename = rubrica_doc_get_name(doc);
  doctype  = rubrica_doc_get_type(doc);

  fs = gtk_file_selection_new(_("Rubrica Save as"));
  mainbox = GTK_FILE_SELECTION (fs)->main_vbox; 

  hbox = gtk_hbox_new(FALSE, 12);
  gtk_box_pack_start(GTK_BOX(mainbox), hbox, FALSE, FALSE, 12);
  gtk_widget_show(hbox);
  
  label = gtk_label_new(_("<b>File type</b>"));
  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);    
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
  gtk_widget_show(label);

  optmenu = rubrica_export_build_options();
  gtk_box_pack_start(GTK_BOX(hbox), optmenu, TRUE, TRUE, 0);
  gtk_option_menu_set_history(GTK_OPTION_MENU(optmenu), SAVE_RUBRICA);   

  gtk_window_set_position(GTK_WINDOW(fs), GTK_WIN_POS_CENTER);
  gtk_window_set_transient_for(GTK_WINDOW(fs), GTK_WINDOW(app));

  g_object_set_data(G_OBJECT(fs), "rubrica", doc);

  if (rubrica_preferences_get_have_default_folder())
    {      
      gchar *path;
     
      path = rubrica_preferences_get_default_folder();
      if (path)
	{
	  gtk_file_selection_set_filename (GTK_FILE_SELECTION(fs), path);

	  g_free(path);
	}
    }
 
  /*    if I have a file name, show it into the entry
  */
  if (filename)
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(fs), filename);
  
  switch (doctype)
    {
    case ADDRESSBOOK_DOC_TYPE:
      g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(fs)->ok_button), "clicked",
		       G_CALLBACK(rubrica_file_card_write_file), fs);
      break;

    case ACCOUNT_DOC_TYPE:
      break;

    default:
      break;
    }
  
  g_signal_connect_swapped(G_OBJECT(GTK_FILE_SELECTION(fs)->cancel_button),
			   "clicked", G_CALLBACK(gtk_widget_destroy), 
			   (gpointer) fs);
  
  gtk_widget_show(fs);
}


void 
rubrica_file_save(RubricaDoc *doc)
{
  GtkWidget *app;
  RubricaDocType doctype;
  gchar *filename;
  gchar *tmp;

  app      = rubrica_get_app();
  filename = rubrica_doc_get_name(doc);
  doctype  = rubrica_doc_get_type(doc);

  /*    call rubrica_file_save_as if filename is NULL
  */
  if (!filename || (g_ascii_strcasecmp(filename, N_("no name")) == 0))
    {    
      rubrica_file_save_as(doc);
    }
  else 
    {
      gchar *name = NULL;

      if (rubrica_preferences_get_have_default_folder())
	{      
	  gchar *path;
	  
	  path = rubrica_preferences_get_default_folder();
	  if (path)
	    {
	      name = g_strdup_printf("%s/%s", path, filename);
	      
	      g_free(path);
	    }
	  else
	    g_print("\nRubrica error ** Can't read default folder");
	}
      else
	name = g_strdup(filename);
      
      switch (doctype)
	{
	case ADDRESSBOOK_DOC_TYPE:
	  if (!rubrica_file_card_write_xmlfile(doc, name))  
	    {      
	      tmp = g_strdup_printf(_("I can't save the file:\n%s"), name);
	      
	      rubrica_dialog_error(tmp);
	      g_free(tmp);
	    }
	  else  /* file has been saved */
	    {
	      rubrica_doc_set_modified(doc, FALSE);
	      // TODO: messaggio nella status bar -- File Salvato con successo
	    }
	  break;
	  
	case  ACCOUNT_DOC_TYPE:
	  break;
	  
	default:
	  break;
	}
      
      g_free(name);
    }
}



/**
 **
 **  Load Routines
 **
 **/

/*  this function is called when user want start rubrica whit a file
*/
RubricaErrorType 
rubrica_file_start_with_file(gchar *file)
{
  gboolean loaded = FALSE;
  RubricaErrorType err = RE_NO_ERROR;

  rubrica_utils_set_start(TRUE);

  /* load file */
  loaded = rubrica_file_load_xmlfile(file, &err);
  if (!loaded)
    {
      rubrica_dialog_error_loading_file(file, err);     
      
      return RE_FILE_NOT_EXIST; 
    }
  
  rubrica_utils_set_start(FALSE);

  return RE_NO_ERROR;
}


void 
rubrica_file_open_file(GtkWidget *w, gpointer data)
{
  GtkWidget *fs = data;
  gchar *filename;
  gchar* msg;
  gboolean loaded;
  RubricaErrorType err;

  filename = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));

  msg = g_strdup_printf("Rubrica *** loading file: %s", filename);
  rubrica_log_write_log(msg);
  g_free(msg);

  loaded = rubrica_file_load_xmlfile(filename, &err);

  if (!loaded)
    rubrica_dialog_error_loading_file(filename, err);
  else
    gtk_widget_destroy(fs);
}


gboolean 
rubrica_file_load_xmlfile(gchar *filename, RubricaErrorType* err)
{
  RubricaDoc *doc;
  gchar *fpath, *fname;
  
  if (!g_file_test(filename, G_FILE_TEST_EXISTS))
    {
      *err = RE_FILE_NOT_EXIST;
            
      return FALSE;
    }

  rubrica_card_add_page_view(filename); 
  doc = rubrica_folder_get_current_document();

  /* real file load 
   */
  if (!rubrica_file_card_real_load_xmlfile(doc, filename, err))
    { 
      return FALSE;
    }

  /* if we are here, the file has been loaded, 
     now we need to put in memory some information 
     about this file and addressbook      
  */
  fpath = g_path_get_dirname(filename);
  fname = g_path_get_basename(filename);
  
  rubrica_doc_set_path(doc, fpath);
  rubrica_doc_set_name(doc, fname);
  rubrica_doc_set_new(doc, FALSE);
  rubrica_doc_set_empty(doc, FALSE);
  rubrica_doc_set_modified(doc, FALSE);

  g_free(fname);
  
  *err = RE_NO_ERROR;
  return TRUE;
}



